{

type token =
    Blank of char
  | Char of char
  | Opening of tag
  | Closing of tag
  | EOF

and tag =
    B | EM | I | PL | S | TT | U
  | Size of char
  | Color of char

exception Lexical_error of string

}

let blank = [' ' '\n' '\r' '\t']
let printable_char = ['!'-';' '=' '?'-'~']

rule token = parse
    blank
     { Blank (Lexing.lexeme_char lexbuf 0) }
  | printable_char
     { Char (Lexing.lexeme_char lexbuf 0) }
  | "</"
     { Closing(tag lexbuf) }
  | "<"
     { Opening(tag lexbuf) }
  | eof
     { EOF }
  | _
     { raise (Lexical_error("Illegal character " ^ 
              String.escaped(Lexing.lexeme lexbuf))) }

and tag = parse
    "B>"
      { B }
  | "EM>"
      { EM }
  | "I>"
      { I }
  | "PL>"
      { PL }
  | "S>"
      { S }
  | "TT>"
      { TT }
  | "U>"
      { U }
  | ['0'-'9'] ">"
      { Size (Lexing.lexeme_char lexbuf 0) }
  | ['r' 'g' 'b' 'c' 'm' 'y' 'k' 'w'] ">"
      { Color (Lexing.lexeme_char lexbuf 0) }
  | _
      { raise (Lexical_error("Illegal tag <" ^
                             String.escaped(Lexing.lexeme lexbuf))) }
